@@ -6,13 +6,13 @@ Rubric::Entry::Query - construct and execute a complex query
 
 =head1 VERSION
 
-version 0.04
+version 0.10
 
- $Id: Query.pm,v 1.7 2005/04/04 22:04:45 rjbs Exp $
+ $Id: Query.pm,v 1.15 2005/06/01 02:02:55 rjbs Exp $
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.10';
 
 =head1 DESCRIPTION
 
@@ -31,19 +31,42 @@ use Rubric::Config;
 
 =head1 METHODS
 
-=head2 query(\%arg)
+=head2 query(\%arg, \%context)
 
 This is the only interface to this module.  Given a hashref of named arguments,
 it returns the entries that match constraints built from the arguments.  It
 generates these constraints with C<get_constraint> and its helpers.  If any
 constraint is invalid, an empty set of results is returned.
 
+The second hashref passed to the method provides context for generating
+implicit query parameters; for example, if the querying user is indicated in
+the context, private entries for that user will be returned.
+
 =cut
 
+sub _private_constraint {
+	my ($self, $user) = @_;
+	my $priv_tag = Rubric::Config->private_tag;
+	   $priv_tag = Rubric::Entry->db_Main->quote($priv_tag);
+
+	return "id NOT IN (SELECT entry FROM entrytags WHERE tag=$priv_tag)"
+		unless $user;
+	
+	$user = Rubric::Entry->db_Main->quote($user);
+	return
+		"((user = $user) OR " .
+		"id NOT IN (SELECT entry FROM entrytags WHERE tag=$priv_tag))";
+}
+
 sub query {
-	my ($self, $arg, $user) = @_;
+	my ($self, $arg, $context) = @_;
+	$context ||= {};
+
 	my @constraints = map { $self->get_constraint($_, $arg->{$_}) } keys %$arg;
 	@constraints = ("1 = 0") if grep { not defined } @constraints;
+	
+	push @constraints, $self->_private_constraint($context->{user})
+		if exists $context->{user};
 
 	$self->get_entries(\@constraints);
 }
@@ -129,11 +152,43 @@ sub constraint_for_exact_tags {
 	return undef unless $tags and ref $tags eq 'ARRAY';
 	return unless my $count = @$tags;
 
+	# XXX determine which one is faster
 	return
 		"(SELECT COUNT(tag) FROM entrytags WHERE entry = entries.id) = $count",
+#		"id IN (SELECT entry FROM entrytags GROUP BY entry HAVING COUNT(tag) = $count)",
 		$self->constraint_for_tags($tags);
 }
 
+=head3 constraint_for_desc_like($value)
+
+=cut
+
+sub constraint_for_desc_like {
+	my ($self, $value) = @_;
+	my $like = substr Rubric::Entry->db_Main->quote($value), 1, -1;
+	"(description LIKE '\%$like\%' OR title LIKE '\%$like\%')"
+}
+
+=head3 constraint_for_body_like($value)
+
+=cut
+
+sub constraint_for_body_like {
+	my ($self, $value) = @_;
+	my $like = substr Rubric::Entry->db_Main->quote($value), 1, -1;
+	"(body LIKE '\%$like\%')"
+}
+
+=head3 constraint_for_like($value)
+
+=cut
+
+sub constraint_for_like {
+	my ($self, $value) = @_;
+	"("  . $self->constraint_for_desc_like($value) .
+	"OR" . $self->constraint_for_body_like($value) . ")"
+}
+
 =head3 constraint_for_has_body($bool)
 
 This returns SQL to limit the results to entries with bodies.
@@ -156,6 +211,20 @@ sub constraint_for_has_link {
 	return $bool ? "link IS NOT NULL" : "link IS NULL";
 }
 
+=head3 constraint_for_first_only($bool)
+
+This returns SQL to limit the results to the first entry posted for any given
+link.
+
+=cut
+
+sub constraint_for_first_only {
+	my ($self, $bool) = @_;
+	return $bool
+		? "(link is NULL OR id IN (SELECT MIN(id) FROM entries GROUP BY link))"
+		: ();
+}
+
 =head3 constraint_for_urimd5($md5)
 
 This returns SQL to limit the results to entries whose link has the given
