@@ -6,13 +6,13 @@ Rubric::WebApp - the web interface to Rubric
 
 =head1 VERSION
 
-version 0.08
+version 0.10
 
- $Id: WebApp.pm,v 1.99 2005/04/07 22:44:42 rjbs Exp $
+ $Id: WebApp.pm,v 1.120 2005/06/01 02:02:55 rjbs Exp $
 
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -62,6 +62,7 @@ use CGI::Application::Session;
 use CGI::Carp qw(fatalsToBrowser);
 
 use Digest::MD5 qw(md5_hex);
+use Encode qw(decode_utf8);
 
 use strict;
 use warnings;
@@ -209,7 +210,7 @@ sub setup {
 	$self->mode_param(path_info => 1);
 	
 	$self->start_mode('login');
-	$self->run_modes([ qw(doc login newuser verify) ]);
+	$self->run_modes([ qw(doc login newuser reset_password verify) ]);
 
 	if ($self->param('current_user') or not Rubric::Config->private_system) {
 		$self->start_mode('entries');
@@ -260,8 +261,14 @@ This displays the single requested entry.
 sub entry {
 	my ($self) = @_;
 
-	return $self->template('no_entry', { reason => 'missing' })
-		unless $self->get_entry->param('entry');
+	my $entry = $self->get_entry;
+
+	return $self->template('no_entry', { reason => 'missing' }) unless $entry;
+
+	return $self->template('no_entry', { reason => 'access' })
+		if  grep { $_ eq Rubric::Config->private_tag } $entry->tags
+		and (not $self->param('current_user')
+		     or  $entry->user ne $self->param('current_user'));
 
 	$self->template('entry_long' => {
 		entry => $self->param('entry'),
@@ -281,7 +288,6 @@ sub get_entry {
 
 	my $entry = Rubric::Entry->retrieve($self->next_path_part);
 	$self->param(entry => $entry);
-	return $self;
 }
 
 =head2 link
@@ -316,7 +322,6 @@ sub get_link {
 	return unless %search;
 	return unless my ($link) = Rubric::Link->search(\%search);
 	$self->param('link', $link);
-	return $self;
 }
 
 =head2 login
@@ -336,14 +341,14 @@ sub login {
 	}
 
 	my $note;
-	if ($self->mode_param ne 'login') {
+	if ($self->get_current_runmode ne 'login') {
 		$note = "You must log in to use this feature.";
 		$self->query->param('then_goto', $self->query->self_url);
 	}
 
 	$self->template('login' => {
 		note      => $note,
-		then_goto => $self->query->param('then_goto'),
+		then_goto => scalar $self->query->param('then_goto'),
 		user      => scalar $self->query->param('user'),
 		user_pending => scalar $self->param('user_pending')
 	});
@@ -364,6 +369,48 @@ sub logout {
 	return $self->redirect_root("Logged out...");
 }
 
+=head2 reset_password
+
+This run mode allows a user to request that his password be reset and emailled
+to him.
+
+=cut
+
+sub reset_password {
+	my ($self) = @_;
+	my $user       = $self->get_user
+	                 || $self->query->param('user')
+	                 && Rubric::User->retrieve($self->query->param('user'));
+	# ^^ allow for username in path or query, though it shouldn't be in path
+	my $reset_code = $self->get_reset_code;
+
+	return $self->template("reset_login") unless $user;
+
+	return $self->setup_reset_code($user) unless $reset_code;
+
+	if (my $password = $user->reset_password($reset_code)) {
+		$self->template("reset", { password => $password });
+	} else {
+		return $self->template("reset_error");
+	}
+
+}
+
+=head2 setup_reset_code
+
+This routine gets a reset code for the user and emails it to him.
+
+=cut
+
+sub setup_reset_code {
+	my ($self, $user) = @_;
+
+	my $reset_code = $user->randomize_reset_code;
+
+	$self->send_reset_email_to($user, $reset_code);
+	$self->template("reset_sent");
+}
+
 =head2 preferences
 
 This method displays account information for the current user.  Some account
@@ -539,17 +586,34 @@ sub create_newuser {
 		email    => $newuser{email},
 	);
 
-	$user{verification_code} = md5_hex("%user".time)
-		unless Rubric::Config->skip_newuser_verification;
-
 	my $user = Rubric::User->create(\%user);
 
-	$self->send_verification_email_to($user)
-		unless Rubric::Config->skip_newuser_verification;
+	unless (Rubric::Config->skip_newuser_verification) {
+		$user->randomize_verification_code;
+		$self->send_verification_email_to($user);
+	}
 
 	$self->template("account_created");
 }
 
+=head2 send_reset_email_to($user)
+
+This method sends an email to the given user with a URI to reset his password.
+
+=cut
+
+sub send_reset_email_to {
+	my ($self, $user) = @_;
+
+	my $message = Rubric::Renderer->process(
+		'reset_mail',
+		'txt',
+		{ user => $user, email_from => Rubric::Config->email_from }
+	);
+
+	send SMTP => $message => Rubric::Config->smtp_server;
+}
+
 =head2 send_verification_email_to($user)
 
 This method sends a verification email to the given user.
@@ -581,14 +645,27 @@ sub verify {
 	return $self->redirect_root("Already logged in...")
 		if $self->param('current_user');
 
-	$self->get_user->get_verification_code;
+	my $user = $self->get_user;
+	my $code = $self->get_verification_code;
 
 	return $self->redirect_root("no such user")
-		if defined $self->param('user') and $self->param('user') eq '';
+		if defined $user and $user eq '';
 
-	return $self->param('user')->verify($self->param('verification_code'))
-		? $self->template('verified')
-		: $self->redirect_root("BAD USER NO VALIDATION");
+	return $user->verify($code) ? $self->template('verified')
+	                            : $self->redirect_root("BAD USER NO VALIDATION");
+}
+
+=head2 get_reset_code
+
+This gets the next part of the path and puts it in the C<reset_code>
+parameter.
+
+=cut
+
+sub get_reset_code {
+	my ($self) = @_;
+
+	$self->param(reset_code => $self->next_path_part);
 }
 
 =head2 get_verification_code
@@ -602,8 +679,6 @@ sub get_verification_code {
 	my ($self) = @_;
 
 	$self->param(verification_code => $self->next_path_part);
-
-	return $self;
 }
 
 =head2 get_user
@@ -616,8 +691,6 @@ sub get_user {
 	my ($self) = @_;
 
 	$self->param(user => Rubric::User->retrieve($self->next_path_part) || '');
-
-	return $self;
 }
 
 =head2 display_entries
@@ -698,6 +771,7 @@ sub render_entries {
 		self_url     => $self->query->self_url(),
 		long_form    => scalar $self->query->param('long_form'),
 		recent_tags  => $self->param('recent_tags'),
+		query_description => $self->param('query_description'),
 	});
 }
 
@@ -712,12 +786,13 @@ sub edit {
 	my ($self) = @_;
 
 	return $self->template('no_entry', { reason => 'missing' })
-		unless $self->get_entry->param('entry');
+		unless $self->get_entry;
 	
 	return $self->template('no_entry', { reason => 'access' })
 		unless $self->param('entry')->user eq $self->param('current_user');
 
 	$self->param('existing_entry', $self->param('entry'));
+	$self->param('existing_link',  $self->param('entry')->link);
 	return $self->post_form();
 }
 
@@ -740,6 +815,15 @@ sub _post_form_contents {
 
 	$form{$_} = $self->query->param($_)
 		for qw(entryid uri title description tags body);
+
+	for (qw(uri title description body tags)) {
+		eval { decode_utf8($form{$_}, Encode::FB_CROAK) };
+		$error{$_} = "Invalid UTF-8 characters in $_." if $@;
+	}
+
+	$form{$_} = $self->query->param($_)
+		for qw(entryid uri title description tags body);
+
 	eval { $form{uri} = URI->new($form{uri})->canonical; };
 	$error{uri} = "Invalid URI" if $@;
 	if (
@@ -751,13 +835,20 @@ sub _post_form_contents {
 		$error{uri} = "Invalid URI; valid schemes are: "
 			. join(" ", @{ Rubric::Config->allowed_schemes });
 	}
+
+	eval { Rubric::Entry->tags_from_string($form{tags}) };
 	$error{tags} =
 		"Tags may only contain letters, numbers, dot, colon, and asterisk."
-		if $form{tags} =~ /[^\s\w\d:.*]/;
+		if $@;
+
+	$error{title} = "You must supply a title." if 
+		$self->query->param('submit')
+		and not length $form{title};
 
 	if ($form{uri} and Rubric::Config->one_entry_per_link) {
 		if (my ($link) = Rubric::Link->search({uri => $form{uri}})) {
-			my $existing_entry = Rubric::Entry->search(
+			$self->param('existing_link', $link);
+			my ($existing_entry) = Rubric::Entry->search(
 				{link => $link, user => $self->param('current_user') }
 			);
 			if ($existing_entry) {
@@ -814,6 +905,7 @@ sub post_form {
 		error          => $error,
 		user           => scalar $self->param('current_user'),
 		existing_entry => scalar $self->param('existing_entry'),
+		existing_link  => scalar $self->param('existing_link'),
 		then_goto      => scalar $self->query->param('then_goto'),
 		when_done      => scalar $self->query->param('when_done'),
 	});
@@ -839,14 +931,14 @@ sub delete {
 		unless $self->get_entry;
 
 	return $self->redirect_root("Not your entry...")
-		unless $self->param('entry')->user eq $self->param('current_user');
+		unless $self->param('entry')->user eq $user;
 
 	$self->param('entry')->delete;
 
-	return $self->redirect(
-		Rubric::WebApp::URI->entries({user=>$self->param('current_user')}),
-		"Deleted..."
-	);
+	my $goto = $self->query->param('then_goto')
+	         || Rubric::WebApp::URI->entries({ user => $user }); 
+
+	return $self->redirect( $goto, "Deleted..." );
 }
 
 =head2 doc
@@ -858,7 +950,8 @@ This runmode returns a mostly-static document from the template path.
 sub doc {
 	my ($self) = @_;
 
-	$self->get_doc->template("docs/" . $self->param('doc_page'));
+	$self->get_doc;
+	$self->template("docs/" . $self->param('doc_page'));
 }
 
 =head2 get_doc
@@ -871,9 +964,8 @@ sub get_doc {
 	my ($self) = @_;
 
 	my $doc_page = $self->next_path_part;
-	$self->param(doc_page => $doc_page) if $doc_page =~ /^\w+$/;
-
-	return $self;
+	return $doc_page =~ /^\w+$/ ? $self->param(doc_page => $doc_page)
+	                            : ();
 }
 
 =head1 TODO
